import os.path
import re
import xml.dom.minidom as minidom

from suds.client import Client

from incidentrecord import IncidentRecord

# Documentation about the xml fields
# http://aproachwstest.muc.amadeus.net:18080/aproachwebservicehelp/jsps/fields.jsp

# PRD URL
WP_WSDL = 'http://aproach.muc.amadeus.net/aproach/services/AproachWebServices?WSDL'
# PPT URL
PPT_WP_WSDL = 'http://aproachwstest.muc.amadeus.net:8095/aproach/services/AproachWebServices?WSDL'


class WinAproachApi:
    def __init__(self, phase='PRD'):
        self.login, password = open(os.path.expanduser('~/.winpwd')).read().split()
        self._wpUrl = WP_WSDL
        if 'PRD' not in phase.upper():
            self._wpUrl = PPT_WP_WSDL
        self._wpclient = Client(self._wpUrl, username=self.login, password=password)
        self.objectMapping = {
            'IR': IncidentRecord
        }

    def update_ir_description(self, ir_id, description_string):
        strRec = ('<record id=\"%s\" type=\"IR\" database=\"prd\"><fft sword=\"S0E01\" name=\"Description\">'
                  '<line>%s</line></fft></record>' % (
                      ir_id, description_string))
        return self._wpclient.service.update(strRec)

    def retrieve(self, record_number):
        xml_string = self._wpclient.service.retrieve(record_number)
        dom = minidom.parseString(xml_string)
        xml_record = dom.getElementsByTagName('record')[0]
        record_type = xml_record.attributes['type'].value
        record = self.objectMapping[record_type]()
        record.recordID(xml_record.attributes['id'].value)
        record.parseXml(xml_string)
        return record

    def search(self, recType, title, startDate, endDate):
        title = re.sub("[/:-]", " ", title)
        search_query = ('<record type="%s" database="@aproachDatabase@"><select><field sword="S0C09"/>'
                        '<field sword="S0BEE"/></select><where><clause>'
                        '((TITL/%s) ((STAC/AA)|(STAC/AC)) (DADT/%s - DADT/%s)</clause></where></record>' % (
                            str(recType).upper(), title, startDate,
                            endDate))
        xml_string = self._wpclient.service.search(search_query)
        dom = minidom.parseString(xml_string)
        xml_record = dom.getElementsByTagName('record')
        return [r.attributes['id'].value for r in xml_record]



from xml.dom.minidom import getDOMImplementation

impl = getDOMImplementation()


class WaList:
    def __init__(self, sword, pword, name):
        self.sword = sword
        self.pword = pword
        self.name = name
        self.items = []

    def __str__(self):
        doc = impl.createDocument(None, "list", None)
        list = doc.documentElement
        list.setAttribute('sword', self.sword)
        list.setAttribute('name', self.name)
        for itemStr in self.items:
            item = doc.createElement('item')
            item.appendChild(doc.createTextNode(itemStr))
            list.appendChild(item)
        return list.toprettyxml()

    def add(self, value):
        self.items.append(value)


class WaFft:
    def __init__(self, sword, pword, name, mandatory):
        self.sword = sword
        self.pword = pword
        self.name = name
        self.mandatory = mandatory == 'Y'
        self.lines = []

    def __str__(self):
        doc = impl.createDocument(None, "fft", None)
        fft = doc.documentElement
        fft.setAttribute('sword', self.sword)
        fft.setAttribute('name', self.name)
        for lineStr in self.lines:
            line = doc.createElement('line')
            line.appendChild(doc.createTextNode(lineStr))
            fft.appendChild(line)
        return fft.toprettyxml()

    def add(self, value):
        self.lines.append(value)

    def getValue(self, index):
        return self.lines[index]

    def getValues(self):
        return self.lines


class WaField:
    def __init__(self, sword, pword, name, mandatory):
        self.sword = sword
        self.pword = pword
        self.name = name
        self.mandatory = mandatory == 'Y'

    def __str__(self):
        doc = impl.createDocument(None, "field", None)
        field = doc.documentElement
        value = doc.createTextNode(self.value)
        field.appendChild(value)
        field.setAttribute("sword", self.sword)
        field.setAttribute("name", self.name)
        return field.toprettyxml()

    def set(self, value):
        self.value = value

    def getValue(self):
        return self.value

    def isValueSet(self):
        try:
            self.value
        except:
            return False
        return True



import json
import os.path

import requests

base_url = 'https://api.github.com'
credentials_file = '~/.ghpwd'


class GitHubApi:
    def __init__(self, repo_owner, repo_name):
        self.repoOwner = repo_owner
        self.repoName = repo_name

    def list_github_issues(self, labels, since):
        login, password = read_credentials()
        '''List an issue on github.com with given labels.'''
        # Our url to list issues via GET
        url = '%s/repos/%s/%s/issues' % (base_url, self.repoOwner, self.repoName)
        # Create an authenticated session to create the issue
        session = requests.session()
        session.auth = (login, password)
        r = session.get(url, params='labels=%s&since=%sT00:00:00Z&state=all' % (labels, since))
        if r.status_code == 200:
            return [i['body'] for i in json.loads(r.content.decode('utf-8'))]
        else:
            print('Could not find Issues for Labels "%s"' % labels)
            print('Response:', r.content)
            return None

    def create_github_issue(self, title, body=None, assignee=None, milestone=None, labels=None):
        login, password = read_credentials()
        '''Create an issue on github.com using the given parameters.'''
        # Our url to create issues via POST
        url = '%s/repos/%s/%s/issues' % (base_url, self.repoOwner, self.repoName)
        # Create an authenticated session to create the issue
        session = requests.session()
        session.auth = (login, password)
        # Create our issue
        issue = {'title': title,
                 'body': body,
                 'assignee': assignee,
                 'milestone': milestone,
                 'labels': labels}
        # Add the issue to our repository
        r = session.post(url, json.dumps(issue))
        if r.status_code == 201:
            return json.loads(r.content.decode('utf-8'))
        else:
            print('Could not create Issue "%s"' % title)
            print('Response:', r.content)

    def get_project_column_id(self, project_name, column_name):
        login, password = open(os.path.expanduser('~/.ghpwd')).read().split()
        url = '%s/repos/%s/%s/projects' % (base_url, self.repoOwner, self.repoName)
        session = requests.session()
        session.auth = (login, password)
        headers = {'Accept': 'application/vnd.github.inertia-preview+json'}
        r = session.get(url, headers=headers)
        result = None
        if r.status_code == 200:
            projects = {project_name: pid for (project_name, pid) in
                        [(i['name'], i['id']) for i in json.loads(r.content.decode('utf-8'))]}
            project_id = projects.get(project_name)
            if project_id is not None:
                url = '%s/projects/%s/columns' % (base_url, project_id)
                r = session.get(url, headers=headers)
                columns = {column_name: cid for (column_name, cid) in
                           [(i['name'], i['id']) for i in json.loads(r.content.decode('utf-8'))]}
                column_id = columns.get(column_name)
                if column_id is not None:
                    result = column_id
                else:
                    print('Could not find Project column %s for Project %s' % (project_name, column_name))
            else:
                print('Could not find Project Project %s' % project_name)
        else:
            print('Could not find Project column %s for Project %s' % (project_name, column_name))
            print('Response:', r.content)
        return result


def add_issue_to_project_column(issue_id, column_id):
    login, password = read_credentials()
    url = '%s/projects/columns/%s/cards' % (base_url, column_id)
    # Create an authenticated session to create the issue
    session = requests.session()
    session.auth = (login, password)
    headers = {'Accept': 'application/vnd.github.inertia-preview+json'}

    card = {'content_id': issue_id,
            'content_type': 'Issue'
            }
    r = session.post(url, json.dumps(card), headers=headers)
    if r.status_code == 201:
        return json.loads(r.content.decode('utf-8'))
    else:
        print('Could not add issue "%s" to project column' % issue_id)
        print('Response:', r.content)


def read_credentials():
    login, password = open(os.path.expanduser(credentials_file)).read().split()
    return login, password



from setuptools import setup

setup(name='gwapy',
      version='0.2',
      description='gwapy',
      url='',
      author='bat-cha',
      author_email='baptiste.chatrain@amadeus.com',
      license='MIT',
      packages=['gwapy'],
      zip_safe=False)